import unittest
from net_models.models.BaseModels import BaseNetCmModel
from net_templates.filters import CustomFilters, AnsibleFilters
from pydantic.error_wrappers import ValidationError


class TestTemplateFiltersBase(unittest.TestCase):

    pass

class TestToVlanRange(TestTemplateFiltersBase):

    TEST_CLASS = CustomFilters()

    def test_from_int_list(self):

        test_cases = [
            {
                "test_name": "Test-From-Int-01",
                "data": list(range(1,4)),
                "result": "1-3"
            },
            {
                "test_name": "Test-From-Int-02",
                "data": [1, 3, 5],
                "result": "1,3,5"
            },
            {
                "test_name": "Test-From-Int-03",
                "data": [1, 2, 3, 4, 5, 7, 8, 9, 11],
                "result": "1-5,7-9,11"
            },
            {
                "test_name": "Test-From-Text-01",
                "data": "all",
                "result": "all"
            },
            {
                "test_name": "Test-From-Text-02",
                "data": "none",
                "result": "none"
            }
        ]
        for test_case in test_cases:
            with self.subTest(msg=test_case["test_name"]):
                want = test_case["result"]
                have = self.TEST_CLASS.to_vlan_range(test_case["data"])
                self.assertEqual(want, have)


class TestToModel(TestTemplateFiltersBase):

    def test_valid_01(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1",
            "key": {
                "encryption_type": 0,
                "value": "SuperSecret"
            }
        }
        model = "RadiusServer"
        model_data = CustomFilters().to_model(data=data, model=model, many=False, serialize=False)
        self.assertIsInstance(model_data, BaseNetCmModel)

    def test_valid_list_01(self):
        data = [
            {
                "name": "Radius-1",
                "server": "192.0.2.1",
                "key": {
                    "encryption_type": 0,
                    "value": "SuperSecret"
                }
            },
            {
                "name": "Radius-2",
                "server": "192.0.2.2",
                "key": {
                    "encryption_type": 0,
                    "value": "SuperSecret"
                }
            }
        ]
        model = "RadiusServer"
        model_data = CustomFilters().to_model(data=data, model=model, many=True, serialize=False)
        if not isinstance(model_data, list):
            self.fail("Model data is not list.")
        elif not all([isinstance(x, BaseNetCmModel) for x in model_data]):
            self.fail("Some elements are not instance of BaseNetCmModel")

    def test_valid_02(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1",
            "key": {
                "encryption_type": 0,
                "value": "SuperSecret"
            }
        }
        model = "RadiusServer"
        model_data = CustomFilters().to_model(data=data, model=model, many=False, serialize=True, dict_params={"exclude_none": True})
        self.assertDictEqual(
            model_data,
            {
                'name': 'Radius-1',
                'server': '192.0.2.1',
                'address_version': 'ipv4',
                'key': {
                    'value': 'SuperSecret',
                    'encryption_type': 0
                }
            }
        )

    def test_invalid_01(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1"
        }
        model = "RadiusServer"
        with self.assertRaises(ValidationError):
            model_data = CustomFilters().to_model(data=data, model=model, many=False, serialize=False)

    def test_invalid_02(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1"
        }
        model = "NonExistentModel"
        with self.assertRaises(ValueError):
            model_data = CustomFilters().to_model(data=data, model=model, many=False, serialize=False)


class TestValidateData(TestTemplateFiltersBase):

    def test_valid_01(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1",
            "key": {
                "encryption_type": 0,
                "value": "SuperSecret"
            }
        }
        model = "RadiusServer"
        self.assertTrue(CustomFilters().validate_data(data=data, model=model))

    def test_invalid_02(self):
        data = {
            "name": "Radius-1",
            "server": "192.0.2.1"
        }
        model = "RadiusServer"
        self.assertFalse(CustomFilters().validate_data(data=data, model=model))

del TestTemplateFiltersBase

if __name__ == '__main__':
    unittest.main()


import unittest
import jinja2
import pathlib
from typing_extensions import Literal
from net_templates.filters import AnsibleFilters, CustomFilters
from net_templates.definitions import TEMPLATES_DIR


class BaseTemplateTest(unittest.TestCase):

    VENDOR = ''
    TEMPLATE_NAME = ''


    @classmethod
    def get_vendor_environment(cls, vendor: Literal["ios"]) -> jinja2.environment.Environment:
        env = jinja2.environment.Environment(
            loader=jinja2.loaders.FileSystemLoader(
                searchpath=TEMPLATES_DIR.joinpath(vendor)
            ),
            autoescape=True,
            trim_blocks=True,
            lstrip_blocks=True,
            undefined=jinja2.ChainableUndefined,
        )

        ansible_filters = AnsibleFilters().filters()
        custom_filters = CustomFilters().filters()
        env.filters.update(ansible_filters)
        env.filters.update(custom_filters)
        return env



import unittest
from tests.tests_ios.BaseTemplateTestIos import BaseTemplateTestIos
from net_models.models.BaseModels.SharedModels import KeyBase
from net_models.models.services.vi.ServerModels import (
    TacacsServer,
    TacacsServerGroup,
    RadiusServer,
    RadiusServerGroup
)

class TestIosTacacsServer(BaseTemplateTestIos):

    TEMPLATE_NAME = 'ios_aaa_server_tacacs'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "params": TacacsServer(
                        name="TACACS-01",
                        server="1.2.3.4",
                        key=KeyBase(
                            encryption_type="0",
                            value="P@ssw0rd"
                        ),
                        single_connection=True
                    )
                },
                "result": (
                    "tacacs server TACACS-01\n"
                    " address ipv4 1.2.3.4\n"
                    " key 0 P@ssw0rd\n"
                    " single-connection\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosTacacsServerGroup(BaseTemplateTestIos):

    TEST_CLASS = TacacsServerGroup
    TEMPLATE_NAME = 'ios_aaa_group_tacacs'

    def test_01(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)


class TestIosTacacsAll(BaseTemplateTestIos):

    TEST_CLASS = TacacsServerGroup
    TEMPLATE_NAME = 'ios_aaa_tacacs_all'

    def test_01(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)


class TestIosRadiusServer(BaseTemplateTestIos):

    TEST_CLASS = RadiusServer
    TEMPLATE_NAME = 'ios_aaa_server_radius'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "params": RadiusServer(
                        name="RADIUS-01",
                        server="192.0.2.1",
                        key=KeyBase(
                            encryption_type="0",
                            value="SecretPassword"
                        ),
                        single_connection=True
                    )
                },
                "result": (
                    "radius server RADIUS-01\n"
                    " address ipv4 192.0.2.1\n"
                    " key 0 SecretPassword\n"
                    " single-connection\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosRadiusServerGroup(BaseTemplateTestIos):

    TEST_CLASS = RadiusServerGroup
    TEMPLATE_NAME = 'ios_aaa_group_radius'

    def test_01(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)


class TestIosRadiusAll(BaseTemplateTestIos):

    TEST_CLASS = RadiusServerGroup
    TEMPLATE_NAME = 'ios_aaa_radius_all'

    def test_01(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)

if __name__ == '__main__':
    unittest.main()


import unittest
import pathlib
import yaml
from tests import BaseTemplateTest
from net_models.models.BaseModels import BaseNetCmModel



class BaseTemplateTestIos(BaseTemplateTest):

    VENDOR = 'ios'
    TEST_CLASS = None
    TEMPLATE_NAME = ''
    RESOURCE_DIR = pathlib.Path(__file__).resolve().parent.parent.joinpath("resources").joinpath(VENDOR)
    ENVIRONMENT = BaseTemplateTest.get_vendor_environment(vendor=VENDOR)

    def common_testbase(self, test_cases: list):

        template = self.ENVIRONMENT.get_template(name=f"{self.TEMPLATE_NAME}.j2")

        for test_case in test_cases:
            with self.subTest(msg=test_case["test_name"]):
                want = test_case["result"]

                dict_params = None
                if "dict_params" in test_case.keys():
                    dict_params = dict_params

                if isinstance(test_case["data"], BaseNetCmModel):
                    if dict_params:
                        test_case["data"] = test_case["data"].dict(**dict_params)
                    else:
                        test_case["data"] = test_case["data"].dict(**dict_params)
                elif isinstance(test_case["data"], dict):
                    for k in test_case["data"].keys():
                        if isinstance(test_case["data"][k], BaseNetCmModel):
                            if dict_params:
                                test_case["data"][k] = test_case["data"][k].serial_dict(**dict_params)
                            else:
                                test_case["data"][k] = test_case["data"][k].dict()

                have = template.render(**test_case["data"])
                print(f"! Template: {self.TEMPLATE_NAME}\tTest: {test_case['test_name']}\n<< BEGIN >>\n{have}<< END >>\n")

                self.assertEqual(want, have)

    def get_test_cases_from_resources(self):
        test_cases = []
        data_files = [x for x in self.RESOURCE_DIR.joinpath("data").iterdir() if x.is_file and x.suffix == ".yml" and x.stem.split("-")[0] == self.TEMPLATE_NAME]
        results_files = [x for x in self.RESOURCE_DIR.joinpath("results").iterdir() if x.is_file and x.suffix == ".txt" and x.stem.split("-")[0] == self.TEMPLATE_NAME]
        data_file_names = [x.stem for x in data_files]
        results_file_names = [x.stem for x in results_files]
        assert set(data_file_names) == set(results_file_names)

        test_cases = [{"test_name": x, "data": None, "result": None} for x in data_file_names]

        for test_case in test_cases:
            if self.TEST_CLASS is not None:
                data = None
                raw_data = yaml.safe_load([x for x in data_files if x.stem == test_case["test_name"]][0].read_text())
                params_keys = [x for x in raw_data.keys() if "params" in x]
                if len(params_keys) == 1:
                    data = dict(raw_data)
                    if isinstance(data[params_keys[0]], dict):
                        data[params_keys[0]] = self.TEST_CLASS.parse_obj(data[params_keys[0]]).serial_dict()
                    elif isinstance(data[params_keys[0]], list):
                        data[params_keys[0]] = [self.TEST_CLASS.parse_obj(x).serial_dict() for x in data[params_keys[0]]]
                else:
                    data = dict(raw_data)
                test_case["data"] = data
                print(test_case["data"])
            else:
                test_case["data"] = yaml.safe_load([x for x in data_files if x.stem == test_case["test_name"]][0].read_text())
            test_case["result"] = [x for x in results_files if x.stem == test_case["test_name"]][0].read_text()
        return test_cases




import ipaddress
from typing import List, Union
from typing_extensions import Literal
from net_templates.filters import BaseFilter
from net_models.models import models_map
import json
import jmespath


class AnsibleFilters(BaseFilter):
    """
    Helper class that mimics some of the Ansible built in filters.

    """

    def ipaddr(self, ip_address: Union[ipaddress.IPv4Address, ipaddress.IPv4Interface, ipaddress.IPv4Network, ipaddress.IPv6Address, ipaddress.IPv6Interface, ipaddress.IPv6Network], operation: str = None):
        address = None
        for func in [ipaddress.ip_address, ipaddress.ip_interface, ipaddress.ip_network]:
            if address is None:
                try:
                    address = func(ip_address)
                except Exception as e:
                    pass
        if operation is None:
            if address:
                return True
            else:
                return False
        if operation == "address":
            if isinstance(address, (ipaddress.IPv4Address, ipaddress.IPv6Address)):
                return address
        raise ValueError("Invalid IP Given")

    def json_query(self, data: Union[list, dict], query: str):
        return jmespath.search(query, data)

    def str_to_obj(self, string: str):
        return eval(string)

    def to_json(self, data: Union[list, dict]) -> str:
        return json.dumps(data)



import unittest
from tests.tests_ios.BaseTemplateTestIos import BaseTemplateTestIos
from net_models.models.services.vi.ServerModels import NtpKey, NtpServer, NtpConfig


class TestIosNtpAuthenticationKey(BaseTemplateTestIos):

    TEST_CLASS = NtpKey
    TEMPLATE_NAME = 'ios_ntp_authentication_key'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "params": NtpKey(
                        key_id=1,
                        encryption_type=0,
                        method="md5",
                        value="SuperSecret",
                        trusted=True
                    )
                },
                "result": (
                    "ntp authentication-key 1 md5 SuperSecret 0\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosNtpTrustedKey(BaseTemplateTestIos):

    TEST_CLASS = NtpKey
    TEMPLATE_NAME = 'ios_ntp_trusted_key'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "params": NtpKey(
                        key_id=1,
                        encryption_type=0,
                        method="md5",
                        value="SuperSecret",
                        trusted=True
                    )
                },
                "result": (
                    "ntp trusted-key 1\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosNtpKeys(BaseTemplateTestIos):

    TEST_CLASS = NtpKey
    TEMPLATE_NAME = 'ios_ntp_keys'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "params": [
                        NtpKey(
                            key_id=1,
                            encryption_type=0,
                            method="md5",
                            value="SuperSecret",
                            trusted=True
                        ),
                        NtpKey(
                            key_id=2,
                            encryption_type=0,
                            method="md5",
                            value="SuperSecret2",
                            trusted=True
                        ),
                        NtpKey(
                            key_id=3,
                            encryption_type=0,
                            method="md5",
                            value="SuperSecret3",
                            trusted=False
                        )
                    ]
                },
                "result": (
                    "ntp authentication-key 1 md5 SuperSecret 0\n"
                    "ntp authentication-key 2 md5 SuperSecret2 0\n"
                    "ntp authentication-key 3 md5 SuperSecret3 0\n"
                    "ntp trusted-key 1\n"
                    "ntp trusted-key 2\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestNtpSession(BaseTemplateTestIos):

    TEST_CLASS = NtpServer
    TEMPLATE_NAME = 'ios_ntp_session'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-Server-01",
                "data": {
                    "params": NtpServer(
                        server="192.0.2.1",
                        vrf="TEST-VRF",
                        prefer=True
                    ),
                    "session_type": "server"
                },
                "result": (
                    "ntp server vrf TEST-VRF 192.0.2.1 prefer\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)

del BaseTemplateTestIos

if __name__ == '__main__':
    unittest.main()


import unittest
from tests.tests_ios.BaseTemplateTestIos import BaseTemplateTestIos
from net_models.models.interfaces.vi import (
    InterfaceCdpConfig,
    InterfaceLldpConfig,
    InterfaceBfdConfig,
    InterfaceModel,
    InterfaceSwitchportModel
)

class TestIosInterfaceL2(BaseTemplateTestIos):

    TEST_CLASS = InterfaceSwitchportModel
    TEMPLATE_NAME = "ios_interface_l2_port"

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-Model-Access-01",
                "data": {
                    "params": InterfaceSwitchportModel(
                        mode="access"
                    )
                },
                "result": (
                    " switchport mode access\n"
                )
            },
            {
                "test_name": "Test-Model-Access-02",
                "data": {
                    "params": InterfaceSwitchportModel(
                        mode="access",
                        untagged_vlan=10
                    )
                },
                "result": (
                    " switchport mode access\n"
                    " switchport access vlan 10\n"
                )
            },
            {
                "test_name": "Test-Model-Trunk-01",
                "data": {
                    "params": InterfaceSwitchportModel(
                        mode="trunk",
                        untagged_vlan=10
                    )
                },
                "result": (
                    " switchport mode trunk\n"
                    " switchport trunk native vlan 10\n"
                )
            },
            {
                "test_name": "Test-Model-Trunk-02",
                "data": {
                    "params": InterfaceSwitchportModel(
                        mode="trunk",
                        encapsulation="dot1q",
                        untagged_vlan=10
                    )
                },
                "result": (
                    " switchport trunk encapsulation dot1q\n"
                    " switchport mode trunk\n"
                    " switchport trunk native vlan 10\n"
                )
            },
            {
                "test_name": "Test-Model-Trunk-03",
                "data": {
                    "params": InterfaceSwitchportModel(
                        mode="trunk",
                        encapsulation="dot1q",
                        untagged_vlan=10,
                        allowed_vlans=[20,21,22,30,31,32,40],
                        negotiation=False
                    )
                },
                "result": (
                    " switchport trunk encapsulation dot1q\n"
                    " switchport mode trunk\n"
                    " switchport trunk native vlan 10\n"
                    " switchport trunk allowed vlan 20-22,30-32,40\n"
                    " switchport nonegotiate\n"
                )
            },
            {
                "test_name": "Test-Model-Trunk-04-With-Defaults",
                "data": {
                    "params": InterfaceSwitchportModel(negotiation=True),
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " no switchport nonegotiate\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)

    def test_02(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)


class TestInterfaceDiscoveryCdp(BaseTemplateTestIos):

    TEST_CLASS = InterfaceCdpConfig
    TEMPLATE_NAME = 'ios_interface_discovery_cdp'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-Enabled-Platform-Default-On_INCLUDE_DEFAULTS-Off",
                "data": {
                    "params": {
                        "enabled": True
                    },
                    "PLATFORM_CDP_DEFAULT_ON": True,
                    "INCLUDE_DEFAULTS": False
                },
                "result": ""
            },
            {
                "test_name": "Test-Enabled-Platform-Default-On_INCLUDE_DEFAULTS-On",
                "data": {
                    "params": {
                        "enabled": True
                    },
                    "PLATFORM_CDP_DEFAULT_ON": True,
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " cdp enable\n"
                )
            },
            {
                "test_name": "Test-Disabled-Platform-Default-On_INCLUDE_DEFAULTS-Off",
                "data": {
                    "params": {
                        "enabled": False
                    },
                    "PLATFORM_CDP_DEFAULT_ON": True,
                    "INCLUDE_DEFAULTS": False
                },
                "result": (
                    " no cdp enable\n"
                )
            },
            {
                "test_name": "Test-Disabled-Platform-Default-On_INCLUDE_DEFAULTS-On",
                "data": {
                    "params": {
                        "enabled": False
                    },
                    "PLATFORM_CDP_DEFAULT_ON": True,
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " no cdp enable\n"
                )
            },
            {
                "test_name": "Test-Enabled-Platform-Default-Off_INCLUDE_DEFAULTS-Off",
                "data": {
                    "params": {
                        "enabled": True
                    },
                    "PLATFORM_CDP_DEFAULT_ON": False,
                    "INCLUDE_DEFAULTS": False
                },
                "result": (
                    " cdp enable\n"
                )
            },
            {
                "test_name": "Test-Enabled-Platform-Default-Off_INCLUDE_DEFAULTS-On",
                "data": {
                    "params": {
                        "enabled": True
                    },
                    "PLATFORM_CDP_DEFAULT_ON": False,
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " cdp enable\n"
                )
            },
            {
                "test_name": "Test-Disabled-Platform-Default-Off_INCLUDE_DEFAULTS-Off",
                "data": {
                    "params": {
                        "enabled": False
                    },
                    "PLATFORM_CDP_DEFAULT_ON": False,
                    "INCLUDE_DEFAULTS": False
                },
                "result": ""
            },
            {
                "test_name": "Test-Disabled-Platform-Default-Off_INCLUDE_DEFAULTS-On",
                "data": {
                    "params": {
                        "enabled": False
                    },
                    "PLATFORM_CDP_DEFAULT_ON": False,
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " no cdp enable\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)



class TestInterfaceDiscoveryLldp(BaseTemplateTestIos):

    TEST_CLASS = InterfaceLldpConfig
    TEMPLATE_NAME = 'ios_interface_discovery_lldp'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test_Transmit-On_Receive-On_Defaults-Off",
                "data": {
                    "params": {
                        "transmit": True,
                        "receive": True
                    },
                    "INCLUDE_DEFAULTS": False
                },
                "result": ""
            },
            {
                "test_name": "Test_Transmit-On_Receive-On_Defaults-On",
                "data": {
                    "params": {
                        "transmit": True,
                        "receive": True
                    },
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " lldp transmit\n"
                    " lldp receive\n"
                )
            },
            {
                "test_name": "Test_Transmit-Off_Receive-Off_Defaults-Off",
                "data": {
                    "params": {
                        "transmit": False,
                        "receive": False
                    },
                    "INCLUDE_DEFAULTS": False
                },
                "result": (
                    " no lldp transmit\n"
                    " no lldp receive\n"
                )
            },
            {
                "test_name": "Test_Transmit-Off_Receive-Off_Defaults-On",
                "data": {
                    "params": {
                        "transmit": False,
                        "receive": False
                    },
                    "INCLUDE_DEFAULTS": True
                },
                "result": (
                    " no lldp transmit\n"
                    " no lldp receive\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosInterfaceBfd(BaseTemplateTestIos):

    TEST_CLASS = InterfaceBfdConfig
    TEMPLATE_NAME = "ios_interface_bfd"

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-Interface-BFD-Template-01",
                "data": {
                    "params": InterfaceBfdConfig(
                        template="BFD-Template-01"
                    )
                },
                "result": (
                    " bfd template BFD-Template-01\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosInterfaceBase(BaseTemplateTestIos):

    TEST_CLASS = InterfaceModel
    TEMPLATE_NAME = "ios_interface_base"

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-Dict-01",
                "data": {
                    "interface": {
                        "name": "Vlan1",
                        "description": "Test"
                    }
                },
                "result": (
                    "interface Vlan1\n"
                    " description Test\n"
                )
            },
            {
                "test_name": "Test-Model-01",
                "data": {
                    "interface": InterfaceModel(name="Vlan1", description="Test")
                },
                "result": (
                    "interface Vlan1\n"
                    " description Test\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


class TestIosInterfaceAll(BaseTemplateTestIos):

    TEMPLATE_NAME = "ios_interfaces"

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-01",
                "data": {
                    "interfaces": {
                        "Vlan1": {
                            "description": "Test"
                        },
                        "Vlan2": {
                            "description": "Test"
                        }
                    }
                },
                "result": (
                    "interface Vlan1\n"
                    " description Test\n"
                    "!\n"
                    "interface Vlan2\n"
                    " description Test\n"
                    "!\n"
                )
            }
        ]
        super().common_testbase(test_cases=test_cases)

del BaseTemplateTestIos

if __name__ == '__main__':
    unittest.main()


from net_models.utils.get_logger import get_logger


class BaseFilter(object):

    def __init__(self):
        self.logger = get_logger(name="NetTemplates-Filter")

    def filters(self):
        filters = {}
        for name, method in self.__class__.__dict__.items():
            if not name.startswith("_") and callable(method):
                filters[name] = getattr(self, name)
        # del filters["filters"]
        return filters


from net_models.models import models_map
from net_models.models.BaseModels import BaseNetCmModel
from pydantic.typing import Union, Dict, List, Literal
from net_templates.filters import BaseFilter


class CustomFilters(BaseFilter):

    def to_model(self, data: Union[Dict, List], model: str, many=True, serialize=True, dict_params=None) -> Union[dict, list]:
        model_data = None
        model_class: BaseNetCmModel = models_map.get(model)
        if model_class is None:
            msg = f"Unknown model: '{model}'. Current models are: '{models_map}'."
            self.logger.error(msg=msg)
            raise ValueError(msg)
        if many is False:
            if isinstance(data, dict):
                if serialize:
                    if dict_params is not None:
                        model_data = model_class.parse_obj(data).serial_dict(**dict_params)
                    else:
                        model_data = model_class.parse_obj(data).serial_dict()
                else:
                    model_data = model_class.parse_obj(data)
            else:
                msg = f"Got unexpected type of data. Expected dict, got {type(data)}."
                self.logger.error(msg=msg)
                raise TypeError(msg)

        elif many is True:
            if isinstance(data, list):
                if serialize:
                    if dict_params is not None:
                        model_data = [model_class.parse_obj(x).serial_dict(**dict_params) for x in data]
                    else:
                        model_data = [model_class.parse_obj(x).serial_dict() for x in data]
                else:
                    model_data = [model_class.parse_obj(x) for x in data]
            else:
                msg = f"Got unexpected type of data. Expected list, got {type(data)}."
                self.logger.error(msg=msg)
                raise TypeError(msg)
        return model_data

    def validate_data(self, data: Union[dict, list], model: str, many=False) -> bool:
        try:
            model_data = self.to_model(data=data, model=model, many=many, serialize=False)
        except Exception as e:
            msg = f"Got Exception while validating data with model 'model'. Exception: {repr(e)}."
            self.logger.error(msg=msg)
            return False
        if model_data is None:
            return False
        elif isinstance(model_data, BaseNetCmModel):
            return True
        elif isinstance(model_data, list):
            if all([isinstance(x, BaseNetCmModel) for x in model_data]):
                return True
            else:
                return False
        else:
            msg = f"Unexpected type - 'model_data' is {type(model_data)}. {model_data}"
            raise TypeError(msg)



    def to_vlan_range(self, vlans: Union[List[int], Literal["none", "all"]]) -> str:

        if isinstance(vlans, str):
            if vlans == "none":
                return "none"
            elif vlans == "all":
                return "all"
            else:
                raise ValueError(f"Invalid string value: {vlans}")
        min_diff = 1
        # Assume vlans is List[int]
        vlans = sorted(list(set(vlans)))
        parts = []
        current_entry = [None, None]
        for vlan in vlans:
            if current_entry[0] is None:
                current_entry[0] = vlan
            elif current_entry[1] is None:
                if current_entry[0] + 1 == vlan:
                    current_entry[1] = vlan
                else:
                    # Flush
                    parts.append(str(current_entry[0]))
                    current_entry = [vlan, None]
            else:
                if current_entry[1] + 1 == vlan:
                    current_entry[1] = vlan
                else:
                    # Flush
                    if (current_entry[1] - current_entry[0]) > min_diff:
                        parts.append(f"{current_entry[0]}-{current_entry[1]}")
                    else:
                        parts.extend(map(str, current_entry))
                    current_entry = [vlan, None]
        # Flush remainder
        if not any(current_entry):
            pass
        elif all(current_entry):
            if (current_entry[1] - current_entry[0]) > min_diff:
                parts.append(f"{current_entry[0]}-{current_entry[1]}")
            else:
                parts.extend(map(str, current_entry))
                current_entry = [None, None]
        else:
            parts.append(str(current_entry[0]))
        vlan_range = ",".join(parts)
        if vlan_range == "":
            return "none"
        elif vlan_range == "1-4094":
            return "all"

        return vlan_range

    def str_to_obj(self, string: str):
        return eval(string)


from setuptools import setup, find_packages

VERSION = "0.1.1"

with open("requirements.txt", "r") as fs:
    reqs = [r for r in fs.read().splitlines() if (len(r) > 0 and not r.startswith("#"))]

setup(
    name="net_templates",
    packages=find_packages(),
    version=VERSION,
    author="Miroslav Hudec <http://github.com/mihudec>",
    description="Network Config Templates",
    install_requires=reqs,
    include_package_data=True
)


import unittest
from tests.tests_ios.BaseTemplateTestIos import BaseTemplateTestIos
from net_models.models.routing.vi.RoutingProtocols import BfdTemplate, BfdAuthentication


class TestIosRoutingBfdTemplate(BaseTemplateTestIos):

    TEST_CLASS = BfdTemplate
    TEMPLATE_NAME = 'ios_routing_bfd_template'

    def test_01(self):
        test_cases = [
            {
                "test_name": "Test-BFD-Template-01",
                "data": {
                    "params": BfdTemplate(
                        name="BFD-Template-01",
                        type="single-hop",
                        min_rx=5,
                        min_tx=5,
                        multiplier=3,
                        authentication=BfdAuthentication(
                            method="md5",
                            keychain="Keychain-01"
                        )
                    )
                },
                "result": (
                    "bfd-template single-hop BFD-Template-01\n"
                    " interval min-tx 5 min-rx 5 multiplier 3\n"
                    " authentication md5 keychain Keychain-01\n"

                )
            },
            {
                "test_name": "Test-BFD-Template-02",
                "data": {
                    "params": BfdTemplate(
                        name="BFD-Template-02",
                        type="single-hop",
                        min_rx=5,
                        min_tx=5,
                        multiplier=3,
                        microseconds=True,
                        authentication=BfdAuthentication(
                            method="md5",
                            keychain="Keychain-01"
                        )
                    )
                },
                "result": (
                    "bfd-template single-hop BFD-Template-02\n"
                    " interval microseconds min-tx 5 min-rx 5 multiplier 3\n"
                    " authentication md5 keychain Keychain-01\n"

                )
            },
            {
                "test_name": "Test-BFD-Template-03",
                "data": {
                    "params": BfdTemplate(
                        name="BFD-Template-03",
                        type="single-hop",
                        both=5,
                        multiplier=3,
                        microseconds=True,
                        authentication=BfdAuthentication(
                            method="md5",
                            keychain="Keychain-01"
                        )
                    )
                },
                "result": (
                    "bfd-template single-hop BFD-Template-03\n"
                    " interval microseconds both 5 multiplier 3\n"
                    " authentication md5 keychain Keychain-01\n"

                )
            }
        ]
        super().common_testbase(test_cases=test_cases)


del BaseTemplateTestIos

if __name__ == '__main__':
    unittest.main()


import unittest
from tests.tests_ios.BaseTemplateTestIos import BaseTemplateTestIos
from net_models.models.routing.vi.BgpModels import BgpNeighborBase, RoutingBgpProcess


class TestIosRoutingBgpProcess(BaseTemplateTestIos):

    TEST_CLASS = RoutingBgpProcess
    TEMPLATE_NAME = 'ios_routing_bgp'

    def test_01(self):
        test_cases = self.get_test_cases_from_resources()
        super().common_testbase(test_cases=test_cases)


del BaseTemplateTestIos

if __name__ == '__main__':
    unittest.main()